from django.apps import AppConfig


class DjangoFirebaseAuthConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_firebase_auth"
    label = "django_firebase_auth"
